<?php

namespace BookneticAddon\MolliePaymentGateway;

use BookneticApp\Providers\Helpers\Math;
use \Mollie\Api\MollieApiClient;
use BookneticApp\Backend\Appointments\Helpers\AppointmentRequestData;
use BookneticApp\Providers\Helpers\Helper;
use BookneticApp\Providers\Common\PaymentGatewayService;
use BookneticApp\Providers\Core\Permission;

class Mollie extends PaymentGatewayService
{

    protected $slug = 'mollie';

    private $_paymentId;
    private $_price;
    private $_currency;
    private $_itemName;
    private $_apiContext;
    private $_successURL;
    public $createPaymentLink;
    private array $_customerInfo = [
        'first_name' => 'undefined',
        'last_name'  => 'undefined',
        'email'      => 'undefined@mail.com'
    ];

    private function getCustomerInfo() : array
    {
        return $this->_customerInfo;
    }

    private function setCustomerInfo( $customerInfo )
    {
        $this->_customerInfo = $customerInfo;
    }

    public function __construct()
    {
        $this->setDefaultTitle(bkntc__('Mollie'));
        $this->setDefaultIcon( MollieAddon::loadAsset('assets/frontend/icons/mollie.svg' ) );

        $this->initMollieSettings();
    }

    public function when( $status, $appointmentRequests = null )
    {
        if( $status && Helper::getOption('hide_confirm_details_step', 'off') == 'on' )
        {
            return false;
        }

        return $status;
    }

    private function initMollieSettings()
    {
        $_apiKey = Helper::getOption('mollie_api_key', '' );

        if( strpos($_apiKey, 'test_') === false && strpos( $_apiKey, 'live_' ) === false )
            return;

        $this->_apiContext = new MollieApiClient();

        try
        {
            $this->_apiContext->setApiKey( $_apiKey );
        }
        catch ( \Exception $ex ) {}
    }

    public function setId( $paymentId )
    {
        $this->_paymentId = ( string ) $paymentId;

        return $this;
    }

    public function getId()
    {
        return $this->_paymentId;
    }

    public function setAmount( $price , $currency = 'USD' )
    {
        $this->_price = $this->normalizePrice( $price, $currency );
        $this->_currency = $currency;

        return $this;
    }

    public function setItemName( $itemName )
    {
        $this->_itemName = $itemName;

        return $this;
    }

    public function setSuccessURL( $url )
    {
        $this->_successURL = $url;

        return $this;
    }

    public function createPaymentUsingSelectedMethods() : array
    {
        $givenName = $this->getCustomerInfo()[ 'first_name' ];
        $familyName = $this->getCustomerInfo()[ 'last_name' ];
        $email = $this->getCustomerInfo()[ 'email' ];

        try {
            $order = $this->_apiContext->orders->create([
                'amount' => [
                    'currency' => $this->_currency,
                    'value'    => $this->_price,
                ],
                'orderNumber' => $this->getId(),
                'locale'      => get_locale(),
                'method'      => Helper::getOption( 'mollie_selected_payment_methods' ),
                'redirectUrl' => $this->_successURL,
                'metadata'    => [
                    'invoice_number' => ( 'SL-' . $this->_paymentId ),
                    'payment_id'     => $this->_paymentId
                ],
                'billingAddress' => ( object ) [
                    'streetAndNumber' => 'undefined',
                    'city'            => 'undefined',
                    'country'         => Helper::getOption( 'mollie_billing_country' ),
                    'postalCode'      => 'undefined',
                    'givenName'       => $givenName,
                    'familyName'      => ! empty ( $familyName ) ? $familyName : 'undefined',
                    'email'           => ! empty ( $email ) ? $email : 'undefined@mail.com',
                ],
                'lines' => [
                    [
                        'name'        => $this->_itemName,
                        'type'        => 'digital',
                        'quantity'    => 1,
                        'vatRate'     => '0.00',
                        'unitPrice'   => [
                            "currency" => $this->_currency,
                            "value"    => $this->_price
                        ],
                        'totalAmount' => [
                            "currency" => $this->_currency,
                            "value"    => $this->_price
                        ],
                        'vatAmount' => [
                            'currency' => $this->_currency,
                            'value'    => '0.00'
                        ]
                    ]
                ]
            ]);

            $approvalUrl    = $order->getCheckoutUrl();
            $paymentId      = $order->id;
            $order          = $this->_apiContext->orders->get( $paymentId );

            $order->redirectUrl = $this->_successURL . "&paymentId=" . $paymentId;

            $order = $order->update();

            return [
                'status'	=> true,
                'url'		=> $approvalUrl
            ];
        }
        catch ( \Exception $ex )
        {
            return [
                'status'	=> false,
                'error'		=> isset($errorData['message']) ? htmlspecialchars($errorData['message']) : bkntc__('Couldn\'t create a payment!')
            ];
        }
    }

    public function createPaymentRequest() : array
    {
        try
        {
            $payment = $this->_apiContext->payments->create([
                "amount" => [
                    "currency"  => $this->_currency,
                    "value"     => $this->_price,
                ],
                "description"   => $this->_itemName,
                "redirectUrl"   => $this->_successURL,
                "metadata"      => [
                    "invoice_number" => ( 'SL-' . $this->_paymentId ),
                    "payment_id" => $this->_paymentId
                ]
            ]);

            $approvalUrl    = $payment->getCheckoutUrl();
            $paymentId      = $payment->id;
            $payment        = $this->_apiContext->payments->get( $paymentId );

            $payment->redirectUrl = $this->_successURL . "&paymentId=" . $paymentId;

            $payment = $payment->update();

            return [
                'status'	=> true,
                'url'		=> $approvalUrl
            ];
        }
        catch (\Exception $ex)
        {
            $errorData = json_encode( $ex );

            return [
                'status'	=> false,
                'error'		=> isset($errorData['message']) ? htmlspecialchars($errorData['message']) : bkntc__('Couldn\'t create a payment!')
            ];
        }
    }

    public function useMollieApi() : array
    {
        if ( Helper::getOption( 'mollie_use_selected_payment_methods' ) )
            return $this->createPaymentUsingSelectedMethods();
        else
            return $this->createPaymentRequest();
    }

    public function check( $paymentId )
    {
        try
        {
            if ( Helper::getOption( 'mollie_use_selected_payment_methods' ) )
                return $this->_apiContext->orders->get( $paymentId );
            else
                return $this->_apiContext->payments->get( $paymentId );
        }
        catch ( \Exception $ex )
        {
            return null;
        }

        return null;
    }

    /**
     * @param AppointmentRequestData $appointmentObj
     * @return object
     */
    public function doPayment( $appointmentRequests )
    {
        $tenantIdParam = ( Helper::isSaaSVersion() ? '&tenant_id=' . Permission::tenantId() : '' );

        $this->setId( $appointmentRequests->paymentId );

        $services = [];

        foreach ( $appointmentRequests->appointments as $appointmentObj )
        {
            $services[] = $appointmentObj->serviceInf->name . ' - ' . Helper::price( $appointmentObj->getPayableToday( true ) );
        }

        $this->setItemName( implode( "\n", $services ) );
        $this->setAmount( $appointmentRequests->getPayableToday(), Helper::getOption( 'currency', 'USD' ) );
        $this->setSuccessURL( site_url() . '/?bkntc_mollie_redirect=1'. $tenantIdParam );

        $this->setCustomerInfo( [
            'first_name' => $appointmentRequests->appointments[ 0 ]->customerData[ 'first_name' ],
            'last_name'  => $appointmentRequests->appointments[ 0 ]->customerData[ 'last_name' ],
            'email'      => $appointmentRequests->appointments[ 0 ]->customerData[ 'email' ]
        ] );

        $res = $this->useMollieApi();

        if( $res[ 'status' ] )
        {
            $responseStatus = true;
            $responseData   = [ 'url' => $res['url'] ];
        }
        else
        {
            $responseStatus = false;
            $responseData   = [ 'error_msg' => $res['error'] ];
        }

        return ( object ) [
            'status'    => $responseStatus,
            'data'      => $responseData
        ];
    }

    public function createPaymentLink( $appointments )
    {
        $tenantIdParam = ( Helper::isSaaSVersion() ? '&tenant_id=' . Permission::tenantId() : '' );

        $appointmentIds = array_map(function ($appointment){
            return $appointment->id;
        },$appointments);
        $appointmentIds = base64_encode(implode(',',$appointmentIds));

        $this->setId( $appointmentIds );

        $services = [];
        $totalAmount = 0;
        foreach ( $appointments as $appointment )
        {
            $totalAmount = Math::add( Math::sub($appointment->total_price , $appointment->paid_amount) , $totalAmount );
            $services[] = $appointment->service_name . ' - ' . Helper::price( Math::sub($appointment->total_price , $appointment->paid_amount) );
        }

        $this->setItemName( implode( "\n", $services ) );
        $this->setAmount( $totalAmount, Helper::getOption( 'currency', 'USD' ) );
        $this->setSuccessURL( site_url() . '/?bkntc_mollie_redirect=1&type=create_payment_link'. $tenantIdParam );

        $this->setCustomerInfo( [
            'first_name' => $appointments[ 0 ]->customer_first_name,
            'last_name'  => $appointments[ 0 ]->customer_last_name,
            'email'      => $appointments[ 0 ]->customer_email
        ] );

        $res = $this->useMollieApi();

        if( $res[ 'status' ] )
        {
            $responseStatus = true;
            $responseData   = [ 'url' => $res['url'] ];
        }
        else
        {
            $responseStatus = false;
            $responseData   = [ 'error_msg' => $res['error'] ];
        }

        return ( object ) [
            'status'    => $responseStatus,
            'data'      => $responseData
        ];
    }

    public function createCustomPayment( $productName, $amount, $callbackUrl )
    {
        $tenantIdParam = ( Helper::isSaaSVersion() ? '&tenant_id=' . Permission::tenantId() : '' );
        $callbackUrl .= strpos( $callbackUrl, '?' ) !== false ? '&' : '?';

        $this->setItemName( $productName );
        $this->setAmount( $amount, Helper::getOption( 'currency', 'USD' ) );

        $this->setSuccessURL( $callbackUrl . 'bkntc_mollie_custom_redirect=1'. $tenantIdParam );

        $res = $this->useMollieApi();

        if( $res[ 'status' ] )
        {
            $responseStatus = true;
            $responseData   = [ 'url' => $res['url'] ];
        }
        else
        {
            $responseStatus = false;
            $responseData   = [ 'error_msg' => $res['error'] ];
        }

        return ( object ) [
            'status'    => $responseStatus,
            'data'      => $responseData
        ];
    }

    public function checkCustomPaymentStatus( $paymentId )
    {
        $isMollieRedirect   = Helper::_get('bkntc_mollie_custom_redirect', false, 'num');
        $molliePaymentId	= Helper::_get('paymentId', '', 'string');

        if ( empty( $isMollieRedirect ) || empty( $molliePaymentId ) )
            return false;

        $mollie = new Mollie();
        $paymentInf = $mollie->check( $molliePaymentId );

        if ( empty( $paymentInf ) || !isset( $paymentInf->metadata->payment_id ) || $paymentInf->metadata->payment_id != $paymentId )
            return false;

        if ($paymentInf->status == 'paid')
        {
            return true;
        }

        return false;
    }

    private function normalizePrice ( $price, $currency )
    {
        $currencies = [
            'BIF' => 0,
            'DJF' => 0,
            'JPY' => 0,
            'KRW' => 0,
            'PYG' => 0,
            'VND' => 0,
            'XAF' => 0,
            'XPF' => 0,
            'CLP' => 0,
            'GNF' => 0,
            'KMF' => 0,
            'MGA' => 0,
            'RWF' => 0,
            'VUV' => 0,
            'XOF' => 0,
            'ISK' => 0,
            'UGX' => 0,
            'UYI' => 0,

            'BHD' => 3,
            'IQD' => 3,
            'JOD' => 3,
            'KWD' => 3,
            'LYD' => 3,
            'OMR' => 3,
            'TND' => 3,
        ];

        if ( array_key_exists( $currency, $currencies ) )
        {
            return number_format( $price, $currencies[ $currency ], '.', '' );
        }
        else
        {
            return number_format( $price, 2, '.', '' );
        }
    }
}
