<?php

namespace BookneticAddon\SquarePaymentGateway;

use BookneticApp\Backend\Appointments\Helpers\AppointmentRequests;
use BookneticApp\Providers\Helpers\Helper;
use BookneticApp\Providers\Common\PaymentGatewayService;
use BookneticApp\Providers\Core\Permission;
use BookneticApp\Providers\Helpers\Math;
use Square\Environment;
use Square\Exceptions\ApiException;
use Square\Models\CheckoutOptions;
use Square\Models\CreatePaymentLinkRequest;
use Square\Models\Money;
use Square\Models\Order;
use Square\Models\OrderLineItem;
use Square\Models\UpdateOrderRequest;
use Square\SquareClient;
use function BookneticAddon\SquarePaymentGateway\bkntc__;

class Square extends PaymentGatewayService
{

    protected $slug = 'square';

    private $order_id;
    private $_apiContext;
    private $_locationId;
    private $_successURL;

    public $createPaymentLink;


    public function __construct(  )
    {
        $this->setDefaultTitle(bkntc__('Square'));
        $this->setDefaultIcon( SquareAddon::loadAsset('assets/frontend/icons/square.svg' ) );

        $this->initSquareSettings();
    }

    public function when( $status, $appointmentRequests = null )
    {
        if( $status && Helper::getOption('hide_confirm_details_step', 'off') == 'on' )
        {
            return false;
        }

        return $status;
    }

    private function initSquareSettings()
    {
        $accessToken	    = Helper::getOption('square_access_token' );
        $locationId	        = Helper::getOption('square_location_id' );
        $mode			    = Helper::getOption('square_mode' ) == 'live' ? 'live' : 'sandbox';
        $this->_locationId  = $locationId;

        $this->_apiContext = new SquareClient([
            'accessToken' => $accessToken,
            'environment' => $mode === 'sandbox'? Environment::SANDBOX : Environment::PRODUCTION,
        ]);
    }

    public function setId( $orderId )
    {
        $this->order_id = $orderId;

        return $this;
    }

    public function setSuccessURL( $url )
    {
        $this->_successURL = $url;

        return $this;
    }

    public function addItem( $itemName , $price , $currency )
    {
        $basePriceMoney = new Money();
        $basePriceMoney->setAmount( (int) Math::mul( $price, 100 ) );
        $basePriceMoney->setCurrency( $currency );

        $orderLineItem = new OrderLineItem(1 );
        $orderLineItem->setName( $itemName );
        $orderLineItem->setBasePriceMoney( $basePriceMoney );
        $this->_items[] = $orderLineItem;
    }

    public function createPaymentRequest()
    {
        $options = new CheckoutOptions();
        $options->setRedirectUrl( $this->_successURL );
        $options->setAskForShippingAddress(false);

        $order = new Order( $this->_locationId );
        $order->setReferenceId( 'SQ-' . $this->order_id );
        $order->setLineItems( $this->_items );
        $order->setMetadata([ 'bkntc_payment_id' => $this->order_id ]);


        $body = new CreatePaymentLinkRequest();
        $body->setIdempotencyKey( 'SQ-' . $this->order_id );
        $body->setOrder( $order );
        $body->setCheckoutOptions( $options );

        try
        {
            $checkoutResponse = $this->_apiContext->getCheckoutApi()->createPaymentLink( $body );

            if( $checkoutResponse->isSuccess() )
            {
                return [ 'status'=>true, 'url' => $checkoutResponse->getResult()->getPaymentLink()->getUrl() ];
            }
            else
            {
                return [
                    'status'	=> false,
                    'error'		=>  htmlspecialchars( $checkoutResponse->getErrors()[0]->getDetail() )
                ];
            }
        }
        catch ( ApiException $e )
        {
            return [
                'status'	=> false,
                'error'		=>  bkntc__( 'Couldn\'t create a payment!' )
            ];
        }
    }

    public function check( $orderId )
    {
        try
        {
            $order = $this->_apiContext->getOrdersApi()->retrieveOrder( $orderId )->getResult()->getOrder();
            $tenders = $order->getTenders();
            $orderMetadata = $order->getMetadata();

            foreach( $order->getFulfillments() as $fulfillment )
            {
                $fulfillment->setState('COMPLETED');
            }

            $paidAmount = 0;

            if ( $tenders )
            {
                foreach ( $tenders AS $tender )
                {
                    $payment = $this->_apiContext
                        ->getPaymentsApi()
                        ->getPayment($tender->getPaymentId())
                        ->getResult()
                        ->getPayment();

                    $status = $payment->getStatus();
                    $amount = $payment->getApprovedMoney()->getAmount();

                    if ( $status === 'COMPLETED' )
                    {
                        $paidAmount = Math::add( $paidAmount, $amount );
                    }
                }
            }

            if( $order->getState() === 'OPEN' && isset( $orderMetadata[ 'bkntc_payment_id' ] ) && $orderMetadata[ 'bkntc_payment_id' ] === $this->order_id )
            {
                $updateOrderRequest = new UpdateOrderRequest();
                $order->setState( 'COMPLETED' );
                $updateOrderRequest->setOrder( $order );

                $this->_apiContext->getOrdersApi()->updateOrder( $order->getId(), $updateOrderRequest );

                return [ 'status' => true ,'total_money'=> $paidAmount ];
            }
            else
            {
                return ['status' => false , 'message' => 'not_approved'];
            }
        }
        catch (\Exception $ex)
        {
            return [ 'status' => false ];
        }
    }

    /**
     * @param AppointmentRequests $appointmentRequests
     * @return object
     */
    public function doPayment( $appointmentRequests )
    {
        $tenantIdParam = ( Helper::isSaaSVersion() ? '&tenant_id=' . Permission::tenantId() : '' );

        $this->setId( $appointmentRequests->paymentId );

        $this->resetItems();

        foreach ($appointmentRequests->appointments as $appointment) {
            $this->addItem(
                $appointment->serviceInf->name,
                $appointment->getPayableToday( true ),
                Helper::getOption('currency', 'USD')
            );
        }

        $this->setSuccessURL(site_url() . '/?booknetic_square_status=success&booknetic_payment_id=' . $appointmentRequests->paymentId  . $tenantIdParam );
        $res = $this->createPaymentRequest();

        if( $res['status'] )
        {
            $responseStatus = true;
            $responseData   = [ 'url' => $res['url'] ];
        }
        else
        {
            $responseStatus = false;
            $responseData   = [ 'error_msg' => $res['error'] ];
        }

        return (object) [
            'status'    => $responseStatus,
            'data'      => $responseData
        ];
    }

    public function createPaymentLink( $appointments )
    {
        $tenantIdParam = ( Helper::isSaaSVersion() ? '&tenant_id=' . Permission::tenantId() : '' );

        $appointmentIds = array_map(function ($appointment){
            return $appointment->id;
        },$appointments);
        $appointmentIds = base64_encode(implode(',',$appointmentIds) . ',' . time());

        $this->setId( $appointmentIds );

        $this->resetItems();

        foreach ($appointments as $appointment) {
            $this->addItem(
                $appointment->service_name,
                Math::sub($appointment->total_price , $appointment->paid_amount),
                Helper::getOption('currency', 'USD')
            );
        }

        $this->setSuccessURL(site_url() . '/?booknetic_square_status=success&type=create_payment_link&booknetic_payment_id=' . $appointmentIds  . $tenantIdParam );
        $res = $this->createPaymentRequest();

        if( $res['status'] )
        {
            $responseStatus = true;
            $responseData   = [ 'url' => $res['url'] ];
        }
        else
        {
            $responseStatus = false;
            $responseData   = [ 'error_msg' => $res['error'] ];
        }

        return (object) [
            'status'    => $responseStatus,
            'data'      => $responseData
        ];
    }

}